import terrapin.*;
import processing.pdf.*;

int XSIZE=int(3.6*297);
int YSIZE=int(3.6*210);

PFont font16, font32;
PGraphicsPDF pdf;

void setup() {
     noLoop();
     size(XSIZE,YSIZE);
     pdf = (PGraphicsPDF) beginRecord(PDF, "CGG_Nr21.pdf");
    //this was tricky, see JunTestFont or email 20-12-2011
}

class PPoint extends PVector {
      PPoint(float x, float y){
            super(x,y);
      }
      //the following method does not change "this" but makes a new point
      //I would have preferred a static method,  but that was not allowed
      //inpol interpolates "this" and B, to give a middle point, half way 
      PPoint intpol(PPoint B){
             PVector M = mult(PPoint.add(this,B),0.5);
             return new PPoint(M.x,M.y);
      }
      PPoint intpol(PPoint B, float f){
             PVector M = PPoint.add(mult(this,1-f),mult(B,f));
             return new PPoint(M.x,M.y);
      }
}

void setPosition(Terrapin t, PPoint P){
     t.x = int(P.x);
     t.y = int(P.y);
}
PPoint getPosition(Terrapin t){
     return new PPoint(t.x,t.y);
}
void setRotation(Terrapin t, int theta){
     t.setRotation(theta);
}
int   getRotation(Terrapin t){
      return t.getRotation();
}
   
class CGG {
      Terrapin t;
      //designed for vertical 
      //glide reflection axis only
      PPoint A,B,C,M,H,I;
      //the following variables can be used after the figure-construction
      //method ABCM has only been executed, useful for arrow directions,
      int i2h;
      //the following variables can be used after the figure-construction
      //method ABCMfourversions has done its work, useful for finding translation vectors;
      PPoint A1,B1,C1,M1,A2,B2,C2,A3,B3,C3,M3,A4,B4,C4;
      CGG(Terrapin trp){
         t=trp;
      }
void ABCM(PPoint inipos,int inidir) {
        // this method uses one arbitrary line A2B and one almost arbitrary line C2M;
        // the construction method uses guarantees a verical axis of glide reflection 
        // when inidir = 0, that is the positive x axis; the procedure encodes the logic
        // of how the lines are positioned with respect to the four special points A,B,C,M.
        // always run this one BEFORe the fourversions method!!
        PPoint A,B,C,M,H,I,Mbis,Mbisbis;
        setPosition(t,inipos);
        setRotation(t,inidir);
        int s = 20; //scale
        A = getPosition(t);
        A2B(t,s,1);
        B = getPosition(t);
        t.setRotation(inidir);
        t.left(180);   
        A2B(t,s,-1);
        C = getPosition(t);
        // in this construction C is always vertically above A
        print("A:    "); print("x = "); print(A.x); print("y = "); print(A.y); println();
        print("B:    "); print("x = "); print(B.x); print("y = "); print(B.y); println();
        print("C:    "); print("x = "); print(C.x); print("y = "); print(C.y); println();
        t.setRotation(inidir);
        M = A.intpol(C);
        C2M(t,s,M,1);
        Mbis = getPosition(t);
        setPosition(t,A);
        setRotation(t,inidir);
        t.left(180);
        C2M(t,s,M,1);
        Mbisbis = getPosition(t);
        /*experimentally adjust CM to make Mbis and Mbisbis to become equal*/
        print("Mbis:    "); print("x = "); print(Mbis.x); print("y = "); print(Mbis.y); println();
        print("Mbisbis: "); print("x = "); print(Mbisbis.x); print("y = "); print(Mbisbis.y); println();
        //add some auxiliary lines
        //just for explanation
        H = new PPoint(C.x,C.y); 
        H.add(PVector.mult(PVector.sub(B,M),0.5));
        I = new PPoint(A.x,A.y);
        I.add(PVector.mult(PVector.sub(B,M),0.5));
        setPosition(t,I);
        setRotation(t,inidir);
        t.left(90);
        I2H(t,s);
        i2h = getRotation(t); 
        //now go for a nice arrow
        setPosition(t,B.intpol(M,.4));
        setRotation(t,i2h);
        t.left(90);
        drwArrowFull(t,s);
        drwA(A,s);
        drwB(B,s);
        drwC(C,s);
        drwM(M,s);
        drwH(H,s);
        drwI(I,s);
      }//end ABCM
      
void ABCMfourversions(PPoint inipos,int inidir) {
        //this procedure is almost like four versions of ABCM;
        //but things are scaled down, and special care is taken
        //not to draw any double line; Each figure is numbered so\
        //the first has points A1,B1,C1,M1, the next A2,B2,C2,M2 and so on.
        //of course some points coincide, for example A2 = C1 and so on.
        //the construction works from right to left.
        setPosition(t,inipos);
        setRotation(t,inidir);
        int s = 5; //scale, either 5,10, or 20
        //make first ABC
        A1 = getPosition(t);
        A2B(t,s,1);
        B1 = getPosition(t);
        t.setRotation(inidir);
        t.left(180);   
        A2B(t,s,-1);
        C1 = getPosition(t);
        t.setRotation(inidir);
        M1 = A1.intpol(C1);
        C2M(t,s,M1,1);
        setPosition(t,A1);
        setRotation(t,inidir);
        t.left(180);
        C2M(t,s,M1,1);
        //add an arrow
        setPosition(t,B1.intpol(M1,.4));
        setRotation(t,i2h);
        t.left(90);
        drwArrowFull(t,s);
        //make ABC version 2
        A2 = C1;
        C2 = A1;
        setPosition(t,A2);
        setRotation(t,inidir);
        t.left(180);
        A2B(t,s,1);
        B2 = getPosition(t);
        setRotation(t,inidir);
        A2B(t,s,-1);
        //add an arrow
        setPosition(t,B2.intpol(M1,.4));
        setRotation(t,i2h);
        t.left(270);
        drwArrowFull(t,s);
        //make version 3
        A3 = B2;
        B3 = C2;
        setPosition(t,B3);
        setRotation(t,inidir);
        t.left(180);
        A2B(t,s,1);
        C3 = getPosition(t);
        t.setRotation(inidir);
        M3 = A3.intpol(C3);
        C2M(t,s,M3,-1);
        setPosition(t,A3);
        setRotation(t,inidir);
        t.left(180);
        C2M(t,s,M3,-1);
        //add an arrow
        setPosition(t,B3.intpol(M3,.4));
        setRotation(t,i2h);
        t.left(90);
        drwArrowOpen(t,s);
        //make version 4
        A4 = C3;
        C4 = A3;
        setPosition(t,A4);
        t.setRotation(inidir);
        t.left(180);   
        A2B(t,s,-1);
        B4 = getPosition(t);
        setRotation(t,inidir);
        A2B(t,s,1);
        //add an arrow
        setPosition(t,B4.intpol(M3,.4));
        setRotation(t,i2h);
        t.left(270);
        drwArrowOpen(t,s);
        //done
        drwA(A1,s);
        drwB(B1,s);
        drwC(C1,s);
        drwM(M1,s);
        drwB(B2,s);
        drwC(C3,s);
        drwM(M3,s);
        drwB(B4,s);
      }//end ABCMfourversions
}// end CGG

void draw() {
     fill(0);
     stroke(0);
     background(255);
     pdf.textMode(SHAPE);
     Terrapin t = new Terrapin(this);
     font16 = createFont("Comic Sans MS",16); 
     font32 = createFont("Comic Sans MS",16); 
     drwIntro();
     CGG cgg = new CGG(t);
     
     //create one big basic figure
     cgg.ABCM(new PPoint(250,650),0);
     
     //create one smaller version
     PPoint origin = new PPoint(675,400);
     cgg.ABCMfourversions(origin,0);
     
     //from this one derive the translation vectors
     //for the entire four-block and repeat by translations;
     //sorry for the double lines;
     PVector T1 = PVector.sub(cgg.A1,cgg.B4);
     PVector T2 = PVector.sub(cgg.A1,cgg.A2);
     for (int i=0;i<2;i++){
         for (int j=0;j<3;j++){
             PPoint T = new PPoint(origin.x,origin.y);
             T.add(PVector.mult(T1,i));
             T.add(PVector.mult(T2,j));
             cgg.ABCMfourversions(T,0);
         }
     }
     endRecord();
}

void A2B(Terrapin t, int scalefactor, int r){
     /* r is -1 means reflected */
     //a kind of zigsaw, fairly arbitrary;
     //make sure not to use any absolute coordinates
     t.setPenColor(0, 0, 0);
     int d=scalefactor;
     strokeWeight(d/8+2);
     int alpha=30;
     t.left(r*alpha);
     t.forward(2*d);
     t.left(r*60);
     t.forward(d);
     t.right(r*60);
     t.forward(d);
     t.right(r*60);
     t.forward(2*d);
     t.left(r*60);
     t.forward(d);
     t.left(r*60);
     t.forward(3*d);
     t.right(r*60);
     t.forward(d);
     t.right(r*60);
     t.forward(4*d);
     t.left(r*60);
     t.forward(d);
     t.left(r*60);
     t.forward(4*d);
     t.right(r*60);
     t.forward(2*d);
     t.right(r*alpha);
}

void C2M(Terrapin t, int scalefactor, PVector M, int r){
     // parameter r = -1 means reflected
     //a kind of zigsaw, somewhat arbitrary;
     //make sure not to use any absolute coordinates;
     //two versions of this make the C line AB around M;
     t.setPenColor(0,0, 0);
     int d = scalefactor;
     //int theta = 127;
     strokeWeight(d/8+2);
     t.right(r*101);
     t.forward(d);
     t.right(r*63);
     for (int i=0; i<8; i++){
         t.forward(d);
         t.left(r*130);
         t.forward(d);
         t.right(r*127);
     }   
     t.forward(d);
     PVector here = getPosition(t);
     float dis = PVector.dist(here,M);
     t.moveToward(int(M.x),int(M.y),int(dis));
     //that is shoot the final stretch towards M 
     //it will not scale well, but no loose ends   
}

void I2H(Terrapin t, int scalefactor){
     // this will be a dashed line;
     //not part of the figure, only for explanation
     t.setPenColor(0,0, 255);
     int d=scalefactor;
     strokeWeight(d/8+1);
     t.drawing=false;
     t.forward(d/2);
     for (int i=0; i<9; i++){
         t.drawing=true;
         t.forward(d);
         t.drawing=false;
         t.forward(d);
     }
     t.drawing=true;
}

void drwIntro(){
     textFont(font32,32);
     text("Nr.21, Basic Type CGG",200,50);
     textFont(font16,16);
     text("Bring the arbitrary line AB by glide reflection until it connects to the position BC, were the angle ABC is arbitrary (axis of glide reflection IH parallel to AC at equal distance from A and B). Close the figure by a C-line CA."
         , 200, 100,700,200); 
     text("Network 666,"
         , 650, 250,700,100); 
     text("4 positionings."
         , 650, 270,700,100); 
}
void drwArrowFull(Terrapin t, int scalefactor){
     //draw a big full arrow
     //s must be prferably arround 20
     t.setPenColor(255 , 0, 0);
     int s=scalefactor;
     strokeWeight(s/4+1);
     //outer arrow
     t.forward(3*s);
     t.left(90);
     t.forward(s/4);
     t.right(110);
     t.forward(s);
     t.right(160);
     t.forward(s);//fill tip
     t.forward(-s);//idem
     t.left(20);
     t.forward(s);
     t.right(110);
     t.forward(s/4);
     t.left(90);
     t.forward(3*s);
     t.right(90);
     t.forward(s/8);
}
void drwArrowOpen(Terrapin t, int scalefactor){
     //draw a big non-filled arrow
     //s must be prferably arround 20
     t.setPenColor(255 , 0, 0);
     int s=scalefactor;
     strokeWeight(s/8);
     t.forward(3*s);
     t.left(90);
     t.forward(s/4);
     t.right(110);
     t.forward(s);
     t.right(160);
     t.left(20);
     t.forward(s);
     t.right(110);
     t.forward(s/4);
     t.left(90);
     t.forward(3*s);
     t.right(90);
     t.forward(s/8);
}

void drwA(PVector p, int scalefactor){
     //to mark the points and write their name;
     //(name writing only only for the big versions);
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(255,0,0);
     fill(255,0,0);
     ellipse(p.x,p.y,d+2,d+2); 
     if (d>5) { textFont(font32,32); text("A",p.x-10,p.y+33);}
}

void drwB(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(255,0,0);
     fill(255,0,0);
     ellipse(p.x,p.y,d+2,d+2); 
     if (d>5) { textFont(font32,32);text("B",p.x,p.y+35); }
}

void drwC(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(255,0,0);
     fill(255,0,0);
     ellipse(p.x,p.y,d+2,d+2); 
     if (d>5) { textFont(font32,32); text("C",p.x-10,p.y-10);}
}

void drwH(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(0,0,255);
     fill(0,0,255);
     if (d>5) {
        ellipse(p.x,p.y,d+2,d+2); 
        textFont(font32,32);
        text("H",p.x+10,p.y+10);
     }
}  

void drwI(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(0,0,255);
     fill(0,0,255);
     if (d>5) {
         ellipse(p.x,p.y,d+2,d+2); 
         textFont(font32,32);
         text("I",p.x+10,p.y+10);
      }  
}
     
void drwM(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(0,0,255);
     fill(0,0,255);
     ellipse(p.x,p.y,d,d); 
     fill(255,255,255);
     ellipse(p.x,p.y,d+2,d+2); 
     stroke(0,0,255);
     fill(0,0,255);
     if (d>5) {textFont(font32,32); text("M",p.x-30,p.y+35); } 
}

void stop(){
     endRecord();
}



